"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.Args = void 0;
const option_1 = require("./option");
/**
 * A wrapper around the parser output for retrieving command arguments.
 */
class Args {
    /**
     * @param parserOutput - The parser output.
     */
    constructor(parserOutput) {
        /**
         * The indices of the ordered tokens already retrieved.
         */
        this.usedIndices = new Set();
        /**
         * The current position in the ordered tokens.
         * Increments from 0.
         */
        this.position = 0;
        this.parserOutput = parserOutput;
        this.positionFromEnd = parserOutput.ordered.length - 1;
    }
    /**
     * Whether all ordered tokens have been used.
     */
    get finished() {
        return this.usedIndices.size === this.parserOutput.ordered.length;
    }
    /**
     * The amount of ordered tokens.
     */
    get length() {
        return this.parserOutput.ordered.length;
    }
    /**
     * The amount of remaining ordered tokens.
     */
    get remaining() {
        return this.parserOutput.ordered.length - this.usedIndices.size;
    }
    /**
     * Retrieves the value of the next unused ordered token.
     * That token will now be consider used.
     * @returns The value if there are tokens left.
     */
    single() {
        if (this.finished) {
            return null;
        }
        while (this.usedIndices.has(this.position)) {
            this.position++;
        }
        this.usedIndices.add(this.position);
        return this.parserOutput.ordered[this.position++].value;
    }
    /**
     * Retrieves the value of the next unused ordered token from the end.
     * That token will now be consider used.
     * @returns The value if there are tokens left.
     */
    singleFromEnd() {
        if (this.finished) {
            return null;
        }
        while (this.usedIndices.has(this.positionFromEnd)) {
            this.positionFromEnd--;
        }
        this.usedIndices.add(this.positionFromEnd);
        return this.parserOutput.ordered[this.positionFromEnd--].value;
    }
    /**
     * Retrieves many unused tokens.
     * @param limit - The limit on the amount of tokens to retrieve; defaults to infinite.
     * @param from - Where to start looking for tokens; defaults to current position.
     * @returns The tokens.
     */
    many(limit = Infinity, from = this.position) {
        const ts = [];
        for (let i = from; i < this.length && ts.length < limit; i++) {
            if (this.usedIndices.has(i)) {
                continue;
            }
            this.usedIndices.add(i);
            ts.push(this.parserOutput.ordered[i]);
        }
        return ts;
    }
    /**
     * Retrieves many unused tokens from the end.
     * Note that the order of retrieved tokens will be the same order as in the ordered tokens list.
     * @param limit - The limit on the amount of tokens to retrieve; defaults to infinite.
     * @param from - Where to start looking for tokens; defaults to current position from end.
     * @returns The tokens.
     */
    manyFromEnd(limit = Infinity, from = this.positionFromEnd) {
        const ts = [];
        for (let i = from; i >= 0 && ts.length < limit; i--) {
            if (this.usedIndices.has(i)) {
                continue;
            }
            this.usedIndices.add(i);
            ts.unshift(this.parserOutput.ordered[i]);
        }
        return ts;
    }
    /**
     * Checks if a flag was given.
     * @param keys - The name(s) of the flag.
     * @returns Whether the flag was given.
     */
    flag(...keys) {
        return keys.some(key => this.parserOutput.flags.has(key));
    }
    /**
     * Gets the last value of an option.
     * @param key - The name(s) of the option.
     * @returns The last value of the option if it was given.
     * When there are multiple names, the last value of the first found name is given.
     */
    option(...keys) {
        for (const key of keys) {
            if (this.parserOutput.options.has(key)) {
                const xs = this.parserOutput.options.get(key);
                if (xs.length !== 0) {
                    return xs[xs.length - 1];
                }
            }
        }
        return null;
    }
    /**
     * Gets all the values of an option.
     * @param keys - The name(s) of the option.
     * @returns The values of the option if it was given.
     */
    options(...keys) {
        let found = false;
        const ys = [];
        for (const key of keys) {
            if (this.parserOutput.options.has(key)) {
                found = true;
                ys.push(...this.parserOutput.options.get(key));
            }
        }
        return found ? ys : null;
    }
    /**
     * Finds and retrieves the first unused token that could be transformed.
     * That token will now be consider used.
     * @typeparam T - Output type.
     * @param f - Gives a pair of whether the transformation worked, and the resulting value.
     * @param from - Where to start looking for tokens; defaults to current position.
     * @returns The resulting value if it was found.
     */
    findMap(f, from = this.position) {
        for (let i = from; i < this.length; i++) {
            if (this.usedIndices.has(i)) {
                continue;
            }
            const x = this.parserOutput.ordered[i];
            const o = f(x.value);
            if (o.exists) {
                this.usedIndices.add(i);
                return option_1.some(o.value);
            }
        }
        return option_1.none();
    }
    /**
     * Filters and retrieves all unused tokens that could be transformed.
     * Those tokens will now be consider used.
     * @typeparam T - Output type.
     * @param f - Gives a pair of whether the transformation worked, and the resulting value.
     * @param limit - The limit on the amount of tokens to retrieve; defaults to infinite.
     * @param from - Where to start looking for tokens; defaults to current position.
     * @returns The resulting values.
     */
    filterMap(f, limit = Infinity, from = this.position) {
        const ys = [];
        for (let i = from; i < this.length && ys.length < limit; i++) {
            if (this.usedIndices.has(i)) {
                continue;
            }
            const x = this.parserOutput.ordered[i];
            const o = f(x.value);
            if (o.exists) {
                this.usedIndices.add(i);
                ys.push(o.value);
            }
        }
        return ys;
    }
}
exports.Args = Args;
//# sourceMappingURL=args.js.map