"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.loop1Async = exports.loopAsync = exports.loop1 = exports.loop = void 0;
const loopAction_1 = require("./loopAction");
const result_1 = require("./result");
/**
 * Runs a loop which continuously gets input and attempts to parse it.
 * The loop strategy used will determine how the loop continues and ends.
 * @typeparam S - Custom state type.
 * @typeparam A - Input type.
 * @typeparam Z - Output type.
 * @typeparam E - Error type.
 * @param intialInput - The first input to parse.
 * @param state - Custom state to thread along the loop.
 * @param strat - The loop strategy to use.
 * @returns Either the parsed value or an error.
 */
function loop(intialInput, state, strat) {
    var _a, _b, _c, _d;
    let inp = intialInput;
    let parsed = strat.parse(inp, state);
    for (;;) {
        switch (parsed.action) {
            case 'finish':
                return result_1.ok(parsed.value);
            case 'fail': {
                const r = (_b = (_a = strat.onParseError) === null || _a === void 0 ? void 0 : _a.call(strat, parsed.error, inp, state)) !== null && _b !== void 0 ? _b : loopAction_1.step_();
                switch (r.action) {
                    case 'finish':
                        return result_1.ok(r.value);
                    case 'fail':
                        return result_1.err(r.error);
                }
            }
        }
        const got = strat.getInput(state);
        switch (got.action) {
            case 'step': {
                inp = got.value;
                parsed = strat.parse(inp, state);
                break;
            }
            case 'finish':
                return result_1.ok(got.value);
            case 'fail': {
                const r = (_d = (_c = strat.onInputError) === null || _c === void 0 ? void 0 : _c.call(strat, got.error, state)) !== null && _d !== void 0 ? _d : loopAction_1.fail(got.error);
                switch (r.action) {
                    case 'finish':
                        return result_1.ok(r.value);
                    case 'fail':
                        return result_1.err(r.error);
                }
            }
        }
    }
}
exports.loop = loop;
/**
 * Runs a loop which continuously gets input and attempts to parse it.
 * The loop strategy used will determine how the loop continues and ends.
 * This variant has no initial input.
 * @typeparam S - Custom state type.
 * @typeparam A - Input type.
 * @typeparam Z - Output type.
 * @typeparam E - Error type.
 * @param state - Custom state to thread along the loop.
 * @param strat - The loop strategy to use.
 * @returns Either the parsed value or an error.
 */
function loop1(state, strat) {
    var _a, _b, _c, _d;
    for (;;) {
        const got = strat.getInput(state);
        switch (got.action) {
            case 'step': {
                const inp = got.value;
                const parsed = strat.parse(inp, state);
                switch (parsed.action) {
                    case 'finish':
                        return result_1.ok(parsed.value);
                    case 'fail': {
                        const r = (_b = (_a = strat.onParseError) === null || _a === void 0 ? void 0 : _a.call(strat, parsed.error, inp, state)) !== null && _b !== void 0 ? _b : loopAction_1.step_();
                        switch (r.action) {
                            case 'finish':
                                return result_1.ok(r.value);
                            case 'fail':
                                return result_1.err(r.error);
                        }
                    }
                }
                break;
            }
            case 'finish':
                return result_1.ok(got.value);
            case 'fail': {
                const r = (_d = (_c = strat.onInputError) === null || _c === void 0 ? void 0 : _c.call(strat, got.error, state)) !== null && _d !== void 0 ? _d : loopAction_1.fail(got.error);
                switch (r.action) {
                    case 'finish':
                        return result_1.ok(r.value);
                    case 'fail':
                        return result_1.err(r.error);
                }
            }
        }
    }
}
exports.loop1 = loop1;
/**
 * Runs a loop which continuously gets input and attempts to parse it.
 * The loop strategy used will determine how the loop continues and ends.
 * This variant of the function is asynchronous using `Promise`.
 * @typeparam S - Custom state type.
 * @typeparam A - Input type.
 * @typeparam Z - Output type.
 * @typeparam E - Error type.
 * @param intialInput - The first input to parse.
 * @param state - Custom state to thread along the loop.
 * @param strat - The loop strategy to use.
 * @returns Either the parsed value or an error.
 */
async function loopAsync(intialInput, state, strat) {
    var _a, _b, _c, _d;
    let inp = intialInput;
    let parsed = await strat.parse(inp, state);
    for (;;) {
        switch (parsed.action) {
            case 'finish':
                return result_1.ok(parsed.value);
            case 'fail': {
                const r = (_b = await ((_a = strat.onParseError) === null || _a === void 0 ? void 0 : _a.call(strat, parsed.error, inp, state))) !== null && _b !== void 0 ? _b : loopAction_1.step_();
                switch (r.action) {
                    case 'finish':
                        return result_1.ok(r.value);
                    case 'fail':
                        return result_1.err(r.error);
                }
            }
        }
        const got = await strat.getInput(state);
        switch (got.action) {
            case 'step': {
                inp = got.value;
                parsed = await strat.parse(inp, state);
                break;
            }
            case 'finish':
                return result_1.ok(got.value);
            case 'fail': {
                const r = (_d = await ((_c = strat.onInputError) === null || _c === void 0 ? void 0 : _c.call(strat, got.error, state))) !== null && _d !== void 0 ? _d : loopAction_1.fail(got.error);
                switch (r.action) {
                    case 'finish':
                        return result_1.ok(r.value);
                    case 'fail':
                        return result_1.err(r.error);
                }
            }
        }
    }
}
exports.loopAsync = loopAsync;
/**
 * Runs a loop which continuously gets input and attempts to parse it.
 * The loop strategy used will determine how the loop continues and ends.
 * This variant has no initial input.
 * This variant of the function is asynchronous using `Promise`.
 * @typeparam S - Custom state type.
 * @typeparam A - Input type.
 * @typeparam Z - Output type.
 * @typeparam E - Error type.
 * @param state - Custom state to thread along the loop.
 * @param strat - The loop strategy to use.
 * @returns Either the parsed value or an error.
 */
async function loop1Async(state, strat) {
    var _a, _b, _c, _d;
    for (;;) {
        const got = await strat.getInput(state);
        switch (got.action) {
            case 'step': {
                const inp = got.value;
                const parsed = await strat.parse(inp, state);
                switch (parsed.action) {
                    case 'finish':
                        return result_1.ok(parsed.value);
                    case 'fail': {
                        const r = (_b = await ((_a = strat.onParseError) === null || _a === void 0 ? void 0 : _a.call(strat, parsed.error, inp, state))) !== null && _b !== void 0 ? _b : loopAction_1.step_();
                        switch (r.action) {
                            case 'finish':
                                return result_1.ok(r.value);
                            case 'fail':
                                return result_1.err(r.error);
                        }
                    }
                }
                break;
            }
            case 'finish':
                return result_1.ok(got.value);
            case 'fail': {
                const r = (_d = await ((_c = strat.onInputError) === null || _c === void 0 ? void 0 : _c.call(strat, got.error, state))) !== null && _d !== void 0 ? _d : loopAction_1.fail(got.error);
                switch (r.action) {
                    case 'finish':
                        return result_1.ok(r.value);
                    case 'fail':
                        return result_1.err(r.error);
                }
            }
        }
    }
}
exports.loop1Async = loop1Async;
//# sourceMappingURL=loop.js.map