"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.Parser = void 0;
const unordered_1 = require("./unordered");
const parserOutput_1 = require("./parserOutput");
/**
 * Parses a list of tokens to separate out flags and options.
 */
class Parser {
    /**
     * @param input - The input tokens.
     */
    constructor(input) {
        this.unorderedStrategy = unordered_1.noStrategy();
        this.position = 0;
        this.input = input;
    }
    /**
     * Sets the strategy for parsing unordered arguments.
     * This can be done in the middle of parsing.
     */
    setUnorderedStrategy(s) {
        this.unorderedStrategy = s;
        return this;
    }
    /**
     * Whether the parser is finished.
     */
    get finished() {
        return this.position >= this.input.length;
    }
    shift(n) {
        this.position += n;
    }
    /**
     * Gets the next parsed tokens.
     */
    next() {
        if (this.finished) {
            return { done: true, value: null };
        }
        const ts = this.processToken();
        if (ts == null) {
            throw new Error('Unexpected end of input (this should never happen).');
        }
        return { done: false, value: ts };
    }
    processToken() {
        return this.pFlag() || this.pOption() || this.pCompactOption() || this.pOrdered();
    }
    pFlag() {
        const t = this.input[this.position];
        const f = this.unorderedStrategy.matchFlag(t.value);
        if (f == null) {
            return null;
        }
        this.shift(1);
        const output = parserOutput_1.emptyOutput();
        output.flags.add(f);
        return output;
    }
    pOption() {
        const t = this.input[this.position];
        const o = this.unorderedStrategy.matchOption(t.value);
        if (o == null) {
            return null;
        }
        this.shift(1);
        const output = parserOutput_1.emptyOutput();
        output.options.set(o, []);
        const n = this.input[this.position];
        if (n == null) {
            return output;
        }
        const bad = (this.unorderedStrategy.matchFlag(n.value)
            || this.unorderedStrategy.matchOption(n.value)
            || this.unorderedStrategy.matchCompactOption(n.value)) != null;
        if (bad) {
            return output;
        }
        this.shift(1);
        const xs = output.options.get(o);
        xs.push(n.value);
        return output;
    }
    pCompactOption() {
        const t = this.input[this.position];
        const o = this.unorderedStrategy.matchCompactOption(t.value);
        if (o == null) {
            return null;
        }
        this.shift(1);
        const output = parserOutput_1.emptyOutput();
        output.options.set(o[0], [o[1]]);
        return output;
    }
    pOrdered() {
        const t = this.input[this.position];
        this.shift(1);
        const output = parserOutput_1.emptyOutput();
        output.ordered.push(t);
        return output;
    }
    [Symbol.iterator]() {
        return this;
    }
    /**
     * Runs the parser.
     */
    parse() {
        return parserOutput_1.mergeOutputs(...this);
    }
}
exports.Parser = Parser;
//# sourceMappingURL=parser.js.map