import { ParserOutput } from './parserOutput';
import { Token } from './tokens';
import { Option } from './option';
/**
 * A wrapper around the parser output for retrieving command arguments.
 */
export declare class Args {
    /**
     * The parser output.
     */
    readonly parserOutput: ParserOutput;
    /**
     * The indices of the ordered tokens already retrieved.
     */
    readonly usedIndices: Set<number>;
    /**
     * The current position in the ordered tokens.
     * Increments from 0.
     */
    position: number;
    /**
     * The current position backwards in the ordered tokens.
     * Decrements from the end.
     */
    positionFromEnd: number;
    /**
     * @param parserOutput - The parser output.
     */
    constructor(parserOutput: ParserOutput);
    /**
     * Whether all ordered tokens have been used.
     */
    get finished(): boolean;
    /**
     * The amount of ordered tokens.
     */
    get length(): number;
    /**
     * The amount of remaining ordered tokens.
     */
    get remaining(): number;
    /**
     * Retrieves the value of the next unused ordered token.
     * That token will now be consider used.
     * @returns The value if there are tokens left.
     */
    single(): string | null;
    /**
     * Retrieves the value of the next unused ordered token from the end.
     * That token will now be consider used.
     * @returns The value if there are tokens left.
     */
    singleFromEnd(): string | null;
    /**
     * Retrieves many unused tokens.
     * @param limit - The limit on the amount of tokens to retrieve; defaults to infinite.
     * @param from - Where to start looking for tokens; defaults to current position.
     * @returns The tokens.
     */
    many(limit?: number, from?: number): Token[];
    /**
     * Retrieves many unused tokens from the end.
     * Note that the order of retrieved tokens will be the same order as in the ordered tokens list.
     * @param limit - The limit on the amount of tokens to retrieve; defaults to infinite.
     * @param from - Where to start looking for tokens; defaults to current position from end.
     * @returns The tokens.
     */
    manyFromEnd(limit?: number, from?: number): Token[];
    /**
     * Checks if a flag was given.
     * @param keys - The name(s) of the flag.
     * @returns Whether the flag was given.
     */
    flag(...keys: string[]): boolean;
    /**
     * Gets the last value of an option.
     * @param key - The name(s) of the option.
     * @returns The last value of the option if it was given.
     * When there are multiple names, the last value of the first found name is given.
     */
    option(...keys: string[]): string | null;
    /**
     * Gets all the values of an option.
     * @param keys - The name(s) of the option.
     * @returns The values of the option if it was given.
     */
    options(...keys: string[]): string[] | null;
    /**
     * Finds and retrieves the first unused token that could be transformed.
     * That token will now be consider used.
     * @typeparam T - Output type.
     * @param f - Gives a pair of whether the transformation worked, and the resulting value.
     * @param from - Where to start looking for tokens; defaults to current position.
     * @returns The resulting value if it was found.
     */
    findMap<T>(f: (x: string) => Option<T>, from?: number): Option<T>;
    /**
     * Filters and retrieves all unused tokens that could be transformed.
     * Those tokens will now be consider used.
     * @typeparam T - Output type.
     * @param f - Gives a pair of whether the transformation worked, and the resulting value.
     * @param limit - The limit on the amount of tokens to retrieve; defaults to infinite.
     * @param from - Where to start looking for tokens; defaults to current position.
     * @returns The resulting values.
     */
    filterMap<T>(f: (x: string) => Option<T>, limit?: number, from?: number): T[];
}
