import { LoopAction } from './loopAction';
import { Result } from './result';
/**
 * A strategy for running an input loop.
 * @typeparam S - Custom state type.
 * @typeparam A - Input type.
 * @typeparam Z - Output type.
 * @typeparam E - Error type.
 */
export interface LoopStrategy<S, A, Z, E> {
    /**
     * Gets new input from somewhere e.g. reading a line.
     * @param state - Custom state.
     * @returns A loop action that can: step with the input; finish with some parsed value; fail due to an error.
     */
    getInput(state: S): LoopAction<A, Z, E>;
    /**
     * Parses given input into the desired type.
     * @param input - The input.
     * @param state - Custom state.
     * @returns A loop action that can: step on; finish with some parsed value; fail due to an error.
     */
    parse(input: A, state: S): LoopAction<null, Z, E>;
    /**
     * Handles error on getting new input.
     * This function intercepts the `fail` case of `getInput`.
     * @param error - The error encountered.
     * @param state - Custom state.
     * @returns A loop action that can: step on; finish with some parsed value; fail due to an error.
     */
    onInputError?(error: E, state: S): LoopAction<null, Z, E>;
    /**
     * Handles error on parsing input.
     * This function intercepts the `fail` case of `parse`.
     * @param error - The error encountered.
     * @param input - The input that could not be parsed.
     * @param state - Custom state.
     * @returns A loop action that can: step on; finish with some parsed value; fail due to an error.
     */
    onParseError?(error: E, input: A, state: S): LoopAction<null, Z, E>;
}
/**
 * A strategy for running an input loop asynchronously via `Promise`.
 * @typeparam S - Custom state type.
 * @typeparam A - Input type.
 * @typeparam Z - Output type.
 * @typeparam E - Error type.
 */
export interface LoopStrategyAsync<S, A, Z, E> {
    /**
         * Gets new input from somewhere e.g. reading a line.
         * @param state - Custom state.
         * @returns A loop action that can: step with the input; finish with some parsed value; fail due to an error.
         */
    getInput(state: S): Promise<LoopAction<A, Z, E>>;
    /**
     * Parses given input into the desired type.
     * @param input - The input.
     * @param state - Custom state.
     * @returns A loop action that can: step on; finish with some parsed value; fail due to an error.
     */
    parse(input: A, state: S): Promise<LoopAction<null, Z, E>>;
    /**
     * Handles error on getting new input.
     * This function intercepts the `fail` case of `getInput`.
     * @param error - The error encountered.
     * @param state - Custom state.
     * @returns A loop action that can: step on; finish with some parsed value; fail due to an error.
     */
    onInputError?(error: E, state: S): Promise<LoopAction<null, Z, E>>;
    /**
     * Handles error on parsing input.
     * This function intercepts the `fail` case of `parse`.
     * @param error - The error encountered.
     * @param input - The input that could not be parsed.
     * @param state - Custom state.
     * @returns A loop action that can: step on; finish with some parsed value; fail due to an error.
     */
    onParseError?(error: E, input: A, state: S): Promise<LoopAction<null, Z, E>>;
}
/**
 * Runs a loop which continuously gets input and attempts to parse it.
 * The loop strategy used will determine how the loop continues and ends.
 * @typeparam S - Custom state type.
 * @typeparam A - Input type.
 * @typeparam Z - Output type.
 * @typeparam E - Error type.
 * @param intialInput - The first input to parse.
 * @param state - Custom state to thread along the loop.
 * @param strat - The loop strategy to use.
 * @returns Either the parsed value or an error.
 */
export declare function loop<S, A, Z, E>(intialInput: A, state: S, strat: LoopStrategy<S, A, Z, E>): Result<Z, E>;
/**
 * Runs a loop which continuously gets input and attempts to parse it.
 * The loop strategy used will determine how the loop continues and ends.
 * This variant has no initial input.
 * @typeparam S - Custom state type.
 * @typeparam A - Input type.
 * @typeparam Z - Output type.
 * @typeparam E - Error type.
 * @param state - Custom state to thread along the loop.
 * @param strat - The loop strategy to use.
 * @returns Either the parsed value or an error.
 */
export declare function loop1<S, A, Z, E>(state: S, strat: LoopStrategy<S, A, Z, E>): Result<Z, E>;
/**
 * Runs a loop which continuously gets input and attempts to parse it.
 * The loop strategy used will determine how the loop continues and ends.
 * This variant of the function is asynchronous using `Promise`.
 * @typeparam S - Custom state type.
 * @typeparam A - Input type.
 * @typeparam Z - Output type.
 * @typeparam E - Error type.
 * @param intialInput - The first input to parse.
 * @param state - Custom state to thread along the loop.
 * @param strat - The loop strategy to use.
 * @returns Either the parsed value or an error.
 */
export declare function loopAsync<S, A, Z, E>(intialInput: A, state: S, strat: LoopStrategyAsync<S, A, Z, E>): Promise<Result<Z, E>>;
/**
 * Runs a loop which continuously gets input and attempts to parse it.
 * The loop strategy used will determine how the loop continues and ends.
 * This variant has no initial input.
 * This variant of the function is asynchronous using `Promise`.
 * @typeparam S - Custom state type.
 * @typeparam A - Input type.
 * @typeparam Z - Output type.
 * @typeparam E - Error type.
 * @param state - Custom state to thread along the loop.
 * @param strat - The loop strategy to use.
 * @returns Either the parsed value or an error.
 */
export declare function loop1Async<S, A, Z, E>(state: S, strat: LoopStrategyAsync<S, A, Z, E>): Promise<Result<Z, E>>;
