/**
 * A strategy to match unordered arguments in parsing.
 */
export interface UnorderedStrategy {
    /**
     * Match a flag.
     * @param s - The string.
     * @returns The name of the flag.
     */
    matchFlag(s: string): string | null;
    /**
     * Match an option.
     * @param s - The string.
     * @returns The name of the option.
     */
    matchOption(s: string): string | null;
    /**
     * Match a compact option.
     * @param s - The string.
     * @returns A pair containing the name of the option and the value.
     */
    matchCompactOption(s: string): [string, string] | null;
}
/**
 * Do not match any unordered argument at all.
 * @returns The strategy.
 */
export declare function noStrategy(): UnorderedStrategy;
/**
 * Match unordered arguments according to conventional syntax.
 * '--flag' is a flag.
 * '--opt=' is an option.
 * '--opt=123' is a compact option.
 * @returns The strategy.
 */
export declare function longStrategy(): UnorderedStrategy;
/**
 * Match unordered arguments according to conventional syntax.
 * '--flag' or '-flag' is a flag.
 * '--opt=' or '-opt=' is an option.
 * '--opt=123' or '-opt=123' is a compact option.
 * @returns The strategy.
 */
export declare function longShortStrategy(): UnorderedStrategy;
/**
 * Match unordered arguments with custom prefix and separator.
 * The prefix is the part the preceeds the key name, e.g. '--' in '--foo'.
 * The separator is the part that delimits the key and value e.g. '=' in '--key=value'.
 * @param prefixes - The prefixes to use for unordered arguments.
 * They should be ordered by length in non-increasing order.
 * @param separators - The symbols to use to separate the key and value in options.
 * They should be ordered by length in non-increasing order.
 * @returns The strategy.
 */
export declare function prefixedStrategy(prefixes: string[], separators: string[]): UnorderedStrategy;
/**
 * Pairing of flag/option names to the words usable for them.
 */
declare type Pairing = Record<string, string[]>;
/**
 * Match unordered arguments according to a record of the names to the list of words in a case-sensitive manner.
 * Prefixes like '--' and separators like '=' should be a part of the word.
 * @param flags - Words usable as flags.
 * @param options - Words usable as options.
 * @param compactOptions - Words usable as the key of compact options.
 * They should be ordered by length in non-increasing order.
 * @returns The strategy.
 */
export declare function exactStrategy(flags: Pairing, options: Pairing, compactOptions: Pairing): UnorderedStrategy;
/**
 * Match unordered arguments according to a record of the names to a list of words in a case-insensitive manner.
 * Prefixes like '--' and separators like '=' should be a part of the word.
 * @param flags - Words usable as flags.
 * @param options - Words usable as options.
 * @param compactOptions - Words usable as the key of compact options.
 * They should be ordered by length in non-increasing order.
 * @param locale - The locale(s) to use to compare case.
 * @returns The strategy.
 */
export declare function caseInsensitiveStrategy(flags: Pairing, options: Pairing, compactOptions: Pairing, locale?: string | string[]): UnorderedStrategy;
export {};
